@@ -2,25 +2,31 @@
 import datetime
 import imghdr
 import json
-from glob import glob
 import os
 import pickle
-from shutil import copyfile
+import sys
 import time
-from typing import TypedDict, Optional, List
 import zipfile
+from glob import glob
+from shutil import copyfile
+from typing import List, Optional
+
+if sys.version_info[0] >= 3 and sys.version_info[1] >= 8:
+    from typing import TypedDict
+else:
+    from typing_extensions import TypedDict
 
-from joblib import Parallel, delayed
-from scipy.spatial.transform import Rotation
 import numpy as np
 import open3d as o3d
 import pandas as pd
 import torch
-from PIL import Image
 import yoco
+from joblib import Parallel, delayed
+from PIL import Image
+from scipy.spatial.transform import Rotation
 from tqdm import tqdm
 
-from cpas_toolbox import pointset_utils, quaternion_utils, camera_utils, utils
+from cpas_toolbox import camera_utils, pointset_utils, quaternion_utils, utils
 from cpas_toolbox.datasets import nocs_utils
 
 
@@ -42,12 +48,13 @@ class NOCSDataset(torch.utils.data.Dataset):
         {root_dir}/camera_composed_depth/...
         {root_dir}/val/...
         {root_dir}/train/...
+        {root_dir}/fixed_real_test_obj_models/...
     Which is easily obtained by downloading all the provided files and extracting them
     into the same directory.
 
     Necessary preprocessing of this data is performed during first initialization per
     and is saved to
-        {root_dir}/csap_toolbox/...
+        {root_dir}/cpas_toolbox/...
     """
 
     num_categories = 7
@@ -147,13 +154,13 @@ class NOCSDataset(torch.utils.data.Dataset):
                 with default_dict. See NOCSDataset.Config for supported keys.
         """
         config = yoco.load_config(config, current_dict=NOCSDataset.default_config)
-        self._root_dir = utils.resolve_path(config["root_dir"])
+        self._root_dir_path = utils.resolve_path(config["root_dir"])
         self._split = config["split"]
         self._check_dirs()
         self._camera_convention = config["camera_convention"]
         self._camera = self._get_split_camera()
         self._preprocess_path = os.path.join(
-            self._root_dir, "csap_toolbox", self._split
+            self._root_dir_path, "cpas_toolbox", self._split
         )
         if not os.path.isdir(self._preprocess_path):
             self._preprocess_dataset()
@@ -166,10 +173,9 @@ class NOCSDataset(torch.utils.data.Dataset):
         self._orientation_repr = config["orientation_repr"]
 
     def _check_dirs(self) -> None:
-        directories = [
-            os.path.join(self._root_dir, "gts"),
-        ]
-        # required folders
+        directories = self._get_dirs()
+
+        # required directories
         if all(os.path.exists(directory) for directory in directories):
             pass
         else:
@@ -177,7 +183,7 @@ class NOCSDataset(torch.utils.data.Dataset):
                 f"NOCS dataset ({self._split} split) not found, do you want to download"
                 " it into the following directory:"
             )
-            print("  ", self._root_dir)
+            print("  ", self._root_dir_path)
             while True:
                 decision = input("(Y/n) ").lower()
                 if decision == "" or decision == "y":
@@ -193,20 +199,24 @@ class NOCSDataset(torch.utils.data.Dataset):
 
         # gts only available for real test and camera val
         if self._split in ["real_test", "camera_val"]:
-            dirs.append(os.path.join(self._root_dir, "gts"))
+            dirs.append(os.path.join(self._root_dir_path, "gts"))
 
-        dirs.append(os.path.join(self._root_dir, "obj_models"))
+        dirs.append(os.path.join(self._root_dir_path, "obj_models"))
+
+        # Fixed object model, need to be downloaded separately
+        if self._split == "real_test":
+            dirs.append(os.path.join(self._root_dir_path, "fixed_real_test_obj_models"))
 
         # full depths for CAMERA
         if self._split in ["camera_val", "camera_train"]:
-            dirs.append(os.path.join(self._root_dir, "camera_full_depths"))
+            dirs.append(os.path.join(self._root_dir_path, "camera_full_depths"))
 
         if self._split == "camera_train":
-            dirs.append(os.path.join(self._root_dir, "train"))
+            dirs.append(os.path.join(self._root_dir_path, "train"))
         elif self._split == "camera_val":
-            dirs.append(os.path.join(self._root_dir, "val"))
+            dirs.append(os.path.join(self._root_dir_path, "val"))
         elif self._split in ["real_train", "real_test"]:
-            dirs.append(os.path.join(self._root_dir, self._split))
+            dirs.append(os.path.join(self._root_dir_path, self._split))
         else:
             raise ValueError(f"Specified split {self._split} is not supported.")
 
@@ -258,6 +268,12 @@ class NOCSDataset(torch.utils.data.Dataset):
                 utils.download(
                     "http://download.cs.stanford.edu/orion/nocs/real_test.zip", zip_path
                 )
+            elif identifier == "fixed_real_test_obj_models":
+                zip_path = os.path.join(download_dir, "fixed_real_test_obj_models.zip")
+                utils.download(
+                    "https://github.com/roym899/pose_and_shape_evaluation/releases/download/v1.0.0/fixed_real_test_obj_models.zip",
+                    zip_path,
+                )
             else:
                 raise ValueError(f"Downloading dir {missing_dir} unsupported.")
             z = zipfile.ZipFile(zip_path)
@@ -298,7 +314,7 @@ class NOCSDataset(torch.utils.data.Dataset):
         One file per sample, which currently means per valid object mask will be
         created.
 
-        Preprocessing will be stored on disk to {root_dir}/csap_toolbox/...
+        Preprocessing will be stored on disk to {root_dir}/cpas_toolbox/...
         This function will not store the preprocessing, so it still has to be loaded
         afterwards.
         """
@@ -339,19 +355,19 @@ class NOCSDataset(torch.utils.data.Dataset):
 
         Some png files have jpg extension. This function fixes these models.
         """
-        glob_pattern = os.path.join(self._root_dir, "**", "*.jpg")
+        glob_pattern = os.path.join(self._root_dir_path, "**", "*.jpg")
         files = glob(glob_pattern, recursive=True)
         for filepath in files:
             what = imghdr.what(filepath)
             if what == "png":
                 print("Fixing: ", filepath)
-                folder, problematic_filename = os.path.split(filepath)
+                obj_dir_path, problematic_filename = os.path.split(filepath)
                 name, _ = problematic_filename.split(".")
                 fixed_filename = f"fixed_{name}.png"
-                fixed_filepath = os.path.join(folder, fixed_filename)
+                fixed_filepath = os.path.join(obj_dir_path, fixed_filename)
 
-                mtl_filepath = os.path.join(folder, "model.mtl")
-                bu_mtl_filepath = os.path.join(folder, "model.mtl.old")
+                mtl_filepath = os.path.join(obj_dir_path, "model.mtl")
+                bu_mtl_filepath = os.path.join(obj_dir_path, "model.mtl.old")
                 copyfile(mtl_filepath, bu_mtl_filepath)
 
                 copyfile(filepath, fixed_filepath)
@@ -457,19 +473,21 @@ class NOCSDataset(torch.utils.data.Dataset):
     def _get_color_files(self) -> list:
         """Return list of paths of color images of the selected split."""
         if self._split == "camera_train":
-            glob_pattern = os.path.join(self._root_dir, "train", "**", "*_color.png")
+            glob_pattern = os.path.join(
+                self._root_dir_path, "train", "**", "*_color.png"
+            )
             return sorted(glob(glob_pattern, recursive=True))
         elif self._split == "camera_val":
-            glob_pattern = os.path.join(self._root_dir, "val", "**", "*_color.png")
+            glob_pattern = os.path.join(self._root_dir_path, "val", "**", "*_color.png")
             return sorted(glob(glob_pattern, recursive=True))
         elif self._split == "real_train":
             glob_pattern = os.path.join(
-                self._root_dir, "real_train", "**", "*_color.png"
+                self._root_dir_path, "real_train", "**", "*_color.png"
             )
             return sorted(glob(glob_pattern, recursive=True))
         elif self._split == "real_test":
             glob_pattern = os.path.join(
-                self._root_dir, "real_test", "**", "*_color.png"
+                self._root_dir_path, "real_test", "**", "*_color.png"
             )
             return sorted(glob(glob_pattern, recursive=True))
         else:
@@ -646,7 +664,7 @@ class NOCSDataset(torch.utils.data.Dataset):
             nocs_transform = gts_data["gt_RTs"][gt_id]
             position = nocs_transform[0:3, 3]
             rot_scale = nocs_transform[0:3, 0:3]
-            nocs_scales = np.sqrt(np.sum(rot_scale ** 2, axis=0))
+            nocs_scales = np.sqrt(np.sum(rot_scale**2, axis=0))
             rotation_matrix = rot_scale / nocs_scales[:, None]
             nocs_scale = nocs_scales[0]
         else:  # camera_train, camera_val, real_train
@@ -683,9 +701,9 @@ class NOCSDataset(torch.utils.data.Dataset):
         Return None if split does not have ground truth information.
         """
         if self._split == "real_test":
-            gts_folder = os.path.join(self._root_dir, "gts", "real_test")
+            gts_dir_path = os.path.join(self._root_dir_path, "gts", "real_test")
         elif self._split == "camera_val":
-            gts_folder = os.path.join(self._root_dir, "gts", "val")
+            gts_dir_path = os.path.join(self._root_dir_path, "gts", "val")
         else:
             return None
 
@@ -693,7 +711,7 @@ class NOCSDataset(torch.utils.data.Dataset):
         split_path = path.split(os.sep)
         number = path[-14:-10]
         gts_filename = f"results_{split_path[-3]}_{split_path[-2]}_{number}.pkl"
-        gts_path = os.path.join(gts_folder, gts_filename)
+        gts_path = os.path.join(gts_dir_path, gts_filename)
         return gts_path
 
     def _get_obj_path(self, meta_row: pd.Series) -> str:
@@ -702,17 +720,24 @@ class NOCSDataset(torch.utils.data.Dataset):
             synset_id = meta_row.iloc[2]
             object_id = meta_row.iloc[3]
             obj_path = os.path.join(
-                self._root_dir,
+                self._root_dir_path,
                 "obj_models",
                 self._split.replace("camera_", ""),
                 synset_id,
                 object_id,
                 "model.obj",
             )
-        elif "real" in self._split:  # REAL mesh
+        elif "real_test" in self._split:  # Fixed REAL test meshes
             object_id = meta_row.iloc[2]
             obj_path = os.path.join(
-                self._root_dir, "obj_models", self._split, object_id + ".obj"
+                self._root_dir_path,
+                "fixed_real_test_obj_models",
+                object_id + ".obj",
+            )
+        elif "real_train" in self._split:  # REAL train mesh (not complete)
+            object_id = meta_row.iloc[2]
+            obj_path = os.path.join(
+                self._root_dir_path, "obj_models", self._split, object_id + ".obj"
             )
         else:
             raise ValueError(f"Specified split {self._split} is not supported.")
